//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 1000 $ 
//                        $Date: 2019-04-23 16:14:31 +0200 (Tue, 23 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit;

import java.util.Map;
import java.util.List;
import java.util.Date;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Collections;

import java.io.Serializable;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.DOMException;

import static de.knipp.rritoolkit.DenicConstants.NS_CONTACT;
import static de.knipp.rritoolkit.DenicConstants.PREFIX_CONTACT;

import static de.knipp.rritoolkit.Utils.parseDateTime;

import static de.knipp.rritoolkit.XmlHelper.asBoolean;
import static de.knipp.rritoolkit.XmlHelper.getElements;
import static de.knipp.rritoolkit.XmlHelper.getFirstElement;
import static de.knipp.rritoolkit.XmlHelper.getFirstElementText;

/**
 *  class that holds contact data, both for requests and responses
 *
 *  @version  $Revision: 1000 $
 *  @author   Klaus Malorny
 */

public class ContactData implements Serializable
{
  /** the handle */
  
  private String handle;
  
  
  /** the type */
  
  private Type type;
  
  
  /** the name */
  
  private String name;
  
  
  /** the organisation */
  
  private List<String> organisation;
  
  
  /** the address */
  
  private List<String> address;
  
  
  /** the postal code */
  
  private String postalCode;
  
  
  /** the city */
  
  private String city;
  
  
  /** the country code */
  
  private String countryCode;
  
  
  /** the e-mail addresses */
  
  private List<String> emailAddresses;
  
  
  /** the URI template */
  
  private String uriTemplate;


  /** the change date */
  
  private Date changed;


  /** flag whether contact is holder of a domain in dispute */

  private boolean disputeReference;
  
  
  /**
   *  constructor
   */
  
  public ContactData ()
  {
    organisation = new ArrayList<> ();
    address = new ArrayList<> ();
    emailAddresses = new ArrayList<> ();
  }
  
  
  
  /**
   *  copy constructor
   *
   *  @param other                the other contact to copy from
   */
  
  public ContactData (ContactData other)
  {
    handle = other.handle;
    type = other.type;
    name = other.name;
    organisation = new ArrayList<> (other.organisation);
    address = new ArrayList<> (other.address);
    postalCode = other.postalCode;
    city = other.city;
    countryCode = other.countryCode;
    emailAddresses = new ArrayList<> (other.emailAddresses);
    uriTemplate = other.uriTemplate;
    changed = other.changed;   // date considered as immutable
    disputeReference = other.disputeReference;
  }
  
  
  
  
  
  /**
   *  convert the object to a KV list as part of a contact request
   *
   *  @param list                the list
   *  @throws EncodeException    on encoding exceptions
   */
  
  public void toList (KVList list)
    throws EncodeException
  {
    if (handle != null) 
      list.add ("handle", handle);
    
    if (type != null) 
      list.add ("type", type.getDenicToken ());
    
    if (name != null) 
      list.add ("name", name);
    
    list.add ("organisation", organisation);
    list.add ("address", address);
    
    if (postalCode != null) 
      list.add ("postalcode", postalCode);
    
    if (city != null) 
      list.add ("city", city);
    
    if (countryCode != null) 
      list.add ("countrycode", countryCode);
    
    list.add ("email", emailAddresses);
    
    if (uriTemplate != null)
      list.add ("uri-template", uriTemplate);
  }

  
  
  /**
   *  parse the given key-value list.
   *
   *  @param list                       the list to parse
   *  @throws ResponseParseException    if the parsing failed
   */
  
  public void fromList (KVList list)
    throws ResponseParseException
  {
    handle = list.getFirst ("handle");
    
    String temp = list.getFirst ("type");
    
    if (temp != null)
    {
      type = Type.fromToken (temp);
      
      if (type == null) 
        throw new ResponseParseException ("invalid type value",
          "type", temp);
    } else
    {
      type = null;
    }
    
    name = list.getFirst ("name");
    
    organisation.clear ();
    organisation.addAll (list.getList ("organisation"));
    
    address.clear ();
    address.addAll (list.getList ("address"));
    
    postalCode = list.getFirst ("postalcode");
    city = list.getFirst ("city");
    countryCode = list.getFirst ("countrycode");
    
    emailAddresses.clear ();
    emailAddresses.addAll (list.getList ("email"));
    
    uriTemplate = list.getFirst ("uri-template");
    
    temp = list.getFirst ("disputereference");
    disputeReference = temp != null && temp.equalsIgnoreCase ("true");
    
    changed = parseDateTime (list.getFirst ("changed"));
  }
  

  /**
   *  parse a single field
   * 
   *  @param node                     the parent node
   *  @param localName                the local name
   *  @return                         the value
   *  @throws DOMException            on DOM exceptions
   */
  
  private String parseField (Node node, String localName)
    throws DOMException
  {
    Element el = getFirstElement (node, NS_CONTACT, localName);
    return el == null ? null : el.getTextContent ();
  }

  
  /**
   *  parse a field consisting of multiple entries
   *
   *  @param node                     the parent node
   *  @param localName                the local name
   *  @param list                     the list to fill
   *  @throws DOMException            on DOM exceptions
   */
  
  private void parseField (Node node, String localName, List<String> list)
    throws DOMException
  {
    list.clear ();

    for (Element elTemp: getElements (node, NS_CONTACT, localName))
      list.add (elTemp.getTextContent ());
  }
    
    
    
  /**
   *  parse the given XML subtree
   *
   *  @param node                     the node containing the contact data
   *  @throws DOMException            on DOM exceptions
   *  @throws ResponseParseException  if the parsing failed
   */
  
  public void fromXml (Element node)
    throws DOMException, ResponseParseException
  {
    Element el = getFirstElement (node, NS_CONTACT, "handle");
    handle = el == null ? null : el.getTextContent ();
    
    el = getFirstElement (node, NS_CONTACT, "type");
    
    if (el != null)
    {
      String temp = el.getTextContent ().trim ();
      type = Type.fromToken (temp);
      
      if (type == null) 
        throw new ResponseParseException ("invalid type", el);
      
    } else
    {
      type = null;
    }
    
    name = parseField (node, "name");
    parseField (node, "organisation", organisation);
    el = getFirstElement (node, NS_CONTACT, "postal");
    
    if (el != null)
    { 
      parseField (el, "address", address);
      postalCode = parseField (el, "postalCode");
      city = parseField (el, "city");
      countryCode = parseField (el, "countryCode");
    
    } else
    {
      address.clear ();
      postalCode = city = countryCode = null;
    }
    
    parseField (node, "email", emailAddresses);
    uriTemplate = parseField (node, "uri-template");

    disputeReference = asBoolean (
      getFirstElementText (node, NS_CONTACT, "disputeReference"), false);

    changed = parseDateTime (getFirstElementText (node, NS_CONTACT, "changed"));
  }

  
  /**
   *  add an element to the builder
   *
   *  @param xb                the builder
   *  @param localName         the local name
   *  @param value             the value (may be {@code null})
   *  @throws DOMException     on DOM exceptions
   */
  
  private void add (XmlBuilder xb, String localName, String value)
    throws DOMException
  {
    if (value == null) 
      return;
    
    xb.elt (NS_CONTACT, PREFIX_CONTACT, localName, value);
  }
  

  /**
   *  add elements for a list to the builder
   *
   *  @param xb                the builder
   *  @param localName         the local name
   *  @param values            the value (may be {@code null})
   *  @throws DOMException     on DOM exceptions
   */
  
  private void add (XmlBuilder xb, String localName, List<String> values)
    throws DOMException
  {
    if (values == null || values.isEmpty ()) 
      return;
    
    for (String value: values)
      xb.elt (NS_CONTACT, PREFIX_CONTACT, localName, value);
  }
  
  
  
  /**
   *  convert the object to XML as part of a contact request
   *
   *  @param node              the node to add the contents to
   *  @throws DOMException     on DOM exceptions
   *  @throws EncodeException  on encode exceptions
   */
  
  public void toXml (Node node)
    throws DOMException, EncodeException
  {
    XmlBuilder xb = new XmlBuilder (node);
    xb.elt (NS_CONTACT, PREFIX_CONTACT, "handle", handle);
    add (xb, "type", type == null ? null : type.getDenicToken ());
    add (xb, "name", name);
    add (xb, "organisation", organisation);
    
    if (!address.isEmpty () || postalCode != null || city != null || 
      countryCode != null)
    {
      xb.es (NS_CONTACT, PREFIX_CONTACT, "postal");
      add (xb, "address", address);
      add (xb, "postalCode", postalCode);
      add (xb, "city", city);
      add (xb, "countryCode", countryCode);
      xb.ee ();
    }
    
    add (xb, "email", emailAddresses);
    add (xb, "uri-template", uriTemplate);
  }
  
  
  
  
  /**
   *  convert the contact to a sting for debugging purposes
   *
   *  @return              a string representation
   */

  @Override
  public String toString ()
  {
    StringBuilder sb = new StringBuilder ();
    sb.append (super.toString ());
    sb.append ("\n    handle: ");
    sb.append (handle);
    sb.append ("\n    type: ");
    sb.append (type);
    sb.append ("\n    name: ");
    sb.append (name);
    
    for (String text: organisation)
    {
      sb.append ("\n    organisation: ");
      sb.append (text);
    }

    for (String text: address)
    {
      sb.append ("\n    address: ");
      sb.append (text);
    }
  
    sb.append ("\n    postal code: ");
    sb.append (postalCode);
    sb.append ("\n    city: ");
    sb.append (city);
    sb.append ("\n    country code: ");
    sb.append (countryCode);

    for (String text: emailAddresses)
    {
      sb.append ("\n    e-mail: ");
      sb.append (text);
    }

    sb.append ("\n    uri-template: ");
    sb.append (uriTemplate);
    sb.append ("\n    disputeReference: ");
    sb.append (disputeReference);
    sb.append ("\n    changed: ");
    sb.append (changed);
    sb.append ("\n");
    
    return sb.toString ();
  }
  
  
  //--- access methods ---------------------------------------------------------
  
  
  
  /**
   *  return the handle
   *
   *  @return                the handle or {@code null} if not set
   */
  
  public String getHandle ()
  {
    return handle;
  }
  
  
  
  /**
   *  set the handle
   *
   *  @param theHandle       the handle, may be {@code null}
   */
  
  public void setHandle (String theHandle)
  {
    handle = theHandle;
  }
  
  
  
  /**
   *  return the contact type 
   *
   *  @return                the contact type or {@code null} if not set
   */
  
  public Type getType ()
  {
    return type;
  }
  
  
  
  /**
   *  set the type
   *
   *  @param theType         the type, may be {@code null}
   */
  
  public void setType (Type theType)
  {
    type = theType;
  }
  
  
  /**
   *  return the name
   *
   *  @return                the name
   */
  
  public String getName ()
  {
    return name;
  }
  
  
  /**
   *  set the name
   *
   *  @param theName         the name
   */
  
  public void setName (String theName)
  {
    name = theName;
  }
  
  
  
  /**
   *  return the organisation as a list of strings
   *
   *  @return                the list (read-only)
   */
  
  public List<String> getOrganisation ()
  {
    return Collections.unmodifiableList (organisation);
  }
  
  
  
  /**
   *  return the organisation as an array
   *
   *  @return               the array
   */
  
  public String[] getOrganisationAsArray ()
  {
    String[] array = new String[organisation.size ()];
    return organisation.toArray (array);
  }
  
  
  /**
   *  set the organisation
   *
   *  @param list               the list of strings to initialize with,
   *                            may be {@code null}
   */
  
  public void setOrganisation (List<String> list)
  {
    organisation.clear ();
    if (list != null) organisation.addAll (list);
  }
  
  
  /**
   *  set the organisation
   *
   *  @param lines       the lines of the organisation, may be {@code null}
   */
  
  public void setOrganisation (String... lines)
  {
    organisation.clear ();

    if (lines != null)
      for (String line: lines) organisation.add (line);
  }  
  

  
  
  
  /**
   *  return the address as a list of strings
   *
   *  @return                the list (read-only)
   */
  
  public List<String> getAddress ()
  {
    return Collections.unmodifiableList (address);
  }
  
  
  
  /**
   *  return the address as an array
   *
   *  @return               the array or {@code null} if not set
   */
  
  public String[] getAddressAsArray ()
  {
    String[] array = new String[address.size ()];
    return address.toArray (array);
  }
  
  
  /**
   *  set the address
   *
   *  @param list               the list of strings to initialize with,
   *                            may be {@code null}
   */
  
  public void setAddress (List<String> list)
  {
    address.clear ();
    
    if (list != null) 
      address.addAll (list);
  }
  
  
  /**
   *  set the address
   *
   *  @param lines       the lines of the address, may be {@code null}
   */
  
  public void setAddress (String... lines)
  {
    address.clear ();
    
    if (lines != null)
      Collections.addAll (address, lines);
  }  
  
  
  
  /**
   *  return the postal code
   *
   *  @return                   the postal code, may be {@code null}
   */
  
  public String getPostalCode ()
  {
    return postalCode;
  }
  
  
  /**
   *  set the postal code
   *
   *  @param code              the postal code, may be {@code null}
   */
  
  public void setPostalCode (String code)
  {
    postalCode = code;
  }
  
  
  
  /**
   *  return the city
   *
   *  @return                the city, may be {@code null}
   */
  
  public String getCity ()
  {
    return city;
  }
  
  
  
  /**
   *  set the city
   *
   *  @param theCity         the city, may be {@code null}
   */
  
  public void setCity (String theCity)
  {
    city = theCity;
  }
  
  
  
  /**
   *  return the country code
   *
   *  @return                the country code, may be {@code null}
   */
  
  public String getCountryCode ()
  {
    return countryCode;
  }
  
  
  /**
   *  set the country code
   *
   *  @param code            the country code, may be {@code null}
   */
  
  public void setCountryCode (String code)
  {
    countryCode = code;
  }
  
  
  
  /**
   *  return the e-mail addresses as a list of strings
   *
   *  @return                the list (read-only)
   */

  public List<String> getEMailAddresses ()
  {
    return Collections.unmodifiableList (emailAddresses);
  }



  /**
   *  return the e-mail addresses as an array
   *
   *  @return               the array
   */

  public String[] getEMailAddressesAsArray ()
  {
    String[] array = new String[emailAddresses.size ()];
    return emailAddresses.toArray (array);
  }


  /**
   *  set the e-mail addresses
   *
   *  @param list               the list of strings to initialize with,
   *                            may be {@code null}
   */

  public void setEMailAddresses (List<String> list)
  {
    emailAddresses.clear ();
    
    if (list != null) 
      emailAddresses.addAll (list);
  }


  /**
   *  set the e-mail addresses
   *
   *  @param addrs       the e-mail addresses, may be {@code null}
   */

  public void setEMailAddresses (String... addrs)
  {
    emailAddresses.clear ();
    
    if (addrs != null)
      Collections.addAll (emailAddresses, addrs);
  }



  /**
   *  return the URI template
   * 
   *  @return               the URI template, may be {@code null}
   */
  
  public String getUriTemplate ()
  {
    return uriTemplate;
  }
  
  
  
  /**
   *  set the URI template
   * 
   *  @param newTemplate    the URI template, may be {@code null}
   */
  
  public void setUriTemplate (String newTemplate)
  {
    uriTemplate = newTemplate;
  }

  
  
  /**
   *  return the change date
   *
   *  @return                      the change date, may be {@code null}
   */
  
  public Date getChangeDate ()
  {
    return changed;
  }
  
  
  
  /**
   *  enumeration for the different contact types
   *
   *  @version  $Revision: 1000 $
   *  @author   Klaus Malorny
   */
  
  public static enum Type
  {
    /** person */

    Person ("PERSON"),


    /** role */

    Role ("ROLE"),


    /** organisation */

    Organisation ("ORG"),
    
    
    /** request */
    
    Request ("REQUEST");



    /** the DENIC token */
    
    private String denicToken;
    
    
    /** map from the token to the type */
    
    private static final Map<String, Type> tokenMap;
    
    
    static
    {
      tokenMap = new HashMap<> ();
      
      for (Type t: values ()) 
        tokenMap.put (t.denicToken.toLowerCase (), t);
    }
    
    /**
     *  constructor
     *
     *  @param token               the token that is used in DENIC messages
     *                             for representing the level
     */
    
    private Type (String token)
    {
      denicToken = token;
    }
 
    
    /**
     *  return the DENIC token
     *
     *  @return                     the token
     */
    
    public String getDenicToken ()
    {
      return denicToken;
    }
    
    
    
    /**
     *  return the type for the given token
     *
     *  @param denicToken           the token
     *  @return                     the type, or {@code null} if the
     *                              token is not found
     */
    
    public static Type fromToken (String denicToken)
    {
      return tokenMap.get (denicToken.toLowerCase ());
    }
  }
  
  
  

}




