//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 998 $ 
//                        $Date: 2019-04-18 15:17:08 +0200 (Thu, 18 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.response.DomainInfoResponse;

import static de.knipp.rritoolkit.DenicConstants.NS_DOMAIN;
import static de.knipp.rritoolkit.DenicConstants.PREFIX_DOMAIN;



/**
 *  request to retrieve information about a given domain
 *
 *  @version  $Revision: 998 $
 *  @author   Klaus Malorny
 */

public class DomainInfoRequest extends DomainSimpleRequest
{
  
  /** flag whether recursion is desired */
  
  private boolean withRecursion;
  
  
  /** the authinfo */
  
  private String authInfo;
  
  
  
  /**
   *  constructor
   */
  
  public DomainInfoRequest ()
  {
    // empty
  }
  
  
  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   *  @param recursion              if {@code true}, full contact data
   *                                is retrieved also
   *  @param theAuthInfo            the authinfo, may be {@code null}
   */
  
  public DomainInfoRequest (String theDomainName, String theEncodedDomainName,
    boolean recursion, String theAuthInfo)
  {
    this (theDomainName, theEncodedDomainName, recursion);
    authInfo = theAuthInfo;
  }
  
  
  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   *  @param recursion              if {@code true}, full contact data
   *                                is retrieved also
   */
  
  public DomainInfoRequest (String theDomainName, String theEncodedDomainName,
    boolean recursion)
  {
    super (theDomainName, theEncodedDomainName);
    withRecursion = recursion;
  }


  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   */
  
  public DomainInfoRequest (String theDomainName, String theEncodedDomainName)
  {
    this (theDomainName, theEncodedDomainName, false);
  }


  
  /**
   *  constructor
   *
   *  @param theDomainName             the domain name
   */
  
  public DomainInfoRequest (String theDomainName)
  {
    this (theDomainName, null);
  }
  
  
  //--- implementation of Request ----------------------------------------------

  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */

  @Override
  public DomainInfoResponse execute (Port port)
    throws IOException
  {
    return (DomainInfoResponse) super.execute (port);
  }
  
  
    
  /**
   *  return the XML element name
   *
   *  @return                      the element name
   */

  @Override
  protected String getRequestElementName ()
  {
    return "info";
  }
  
  

  /**
   *  allow the addition of attributes to the request.
   *
   *  @param xb                   the XML builder
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */
  
  @Override
  protected void addAttributes (XmlBuilder xb)
    throws DOMException, EncodeException
  {
    xb.attf ("recursive", withRecursion);
  }
  
  /**
   *  allow the addition of elements within the request.
   * 
   *  @param xb                   the XML builder
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */
  
  @Override
  protected void addElements (XmlBuilder xb)
    throws DOMException, EncodeException
  {
    xb.elt (NS_DOMAIN, PREFIX_DOMAIN, "authinfo", authInfo);
  }
  
  
  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "info";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException       if the encoding fails for some reason
   */
  
  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    super.requestToKVList (list);
    list.set ("recursive", withRecursion ? "true" : "false");
    
    if (authInfo != null)
      list.set ("authinfo", authInfo);
  }
  
  
  
  
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                 the representation
   */
  
  @Override
  public String toString ()
  {
    return super.toString () + "  with recursion: " + withRecursion +
      "\n  authinfo: " + authInfo + "\n";
  }
  
  
  //--- access methods ---------------------------------------------------------
  
  
  
  /**
   *  return whether contact data shall be retrieved also
   *
   *  @return                      {@code true} if so
   */
  
  public boolean withRecursion ()
  {
    return withRecursion;
  }
  
  
  /**
   *  set whether contact data shall be retrieved also
   *
   *  @param recursion             {@code true} if contact data shall be
   *                               retrieved
   */
  
  public void setWithRecursion (boolean recursion)
  {
    withRecursion = recursion;
  }
  
  
  
  /**
   *  return the authinfo
   * 
   *  @return               the authinfo or {@code null} if not set
   */
  
  public String getAuthInfo ()
  {
    return authInfo;
  }
  
  
  
  /**
   *  set the authinfo
   * 
   *  @param theAuthInfo    the authinfo, may be {@code null}
   */
  
  public void setAuthInfo (String theAuthInfo)
  {
    authInfo = theAuthInfo;
  }
}


