//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 937 $ 
//                        $Date: 2017-10-09 17:29:32 +0200 (Mo, 09 Okt 2017) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.response.DomainTransitResponse;



/**
 *  class that represents a domain transit request
 *
 *  @version  $Revision: 937 $
 *  @author   Klaus Malorny
 */

public class DomainTransitRequest extends DomainSimpleRequest
{
  
  /** flag whether the domain shall be disconnected */
  
  private boolean disconnect;
  
  
  
  /**
   *  constructor
   */
  
  public DomainTransitRequest ()
  {
    // empty
  }
  
  
  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   *  @param disconnectDomain       if {@code true} the domain is
   *                                disconnected on execution
   */
  
  public DomainTransitRequest (String theDomainName, 
    String theEncodedDomainName, boolean disconnectDomain)
  {
    super (theDomainName, theEncodedDomainName);
    disconnect = disconnectDomain;
  }


  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII-only domains
   */
  
  public DomainTransitRequest (String theDomainName, 
    String theEncodedDomainName)
  {
    this (theDomainName, theEncodedDomainName, false);
  }


  
  /**
   *  constructor
   *
   *  @param theDomainName             the domain name
   */
  
  public DomainTransitRequest (String theDomainName)
  {
    this (theDomainName, null);
  }
  
  
  //--- implementation of Request ----------------------------------------------
  
  

  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */

  @Override
  public DomainTransitResponse execute (Port port)
    throws IOException
  {
    return (DomainTransitResponse) super.execute (port);
  }

  
    
  /**
   *  return the XML element name
   *
   *  @return                      the element name
   */

  @Override
  protected String getRequestElementName ()
  {
    return "transit";
  }
  
  

  /**
   *  allow the addition of attributes to the request. The default 
   *  implementation is empty and should not be called by implementing 
   *  subclasses
   *
   *  @param xb                   the XML builder
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void addAttributes (XmlBuilder xb)
    throws DOMException, EncodeException
  {
    xb.attf ("disconnect", disconnect);
  }
  
  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "transit";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException       if the encoding fails for some reason
   */

  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    super.requestToKVList (list);
    list.set ("disconnect", disconnect ? "true" : "false");
  }
  
  
  
  
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                 the representation
   */
  
  @Override
  public String toString ()
  {
    return super.toString () + "  disconnect: " + disconnect + "\n";
  }
  
  
  //--- access methods ---------------------------------------------------------
  
  /**
   *  return whether the domain shall be disconnected or not
   *
   *  @return                      {@code true} if the domain shall be
   *                               disconnected
   */
  
  public boolean disconnect ()
  {
    return disconnect;
  }
  
  
  /**
   *  set whether the domain shall be disconnected or not
   *
   *  @param disconnectDomain      {@code true} if the domain shall be
   *                               disconnected
   */
  
  public void setDisconnect (boolean disconnectDomain)
  {
    disconnect = disconnectDomain;
  }
  

}


