//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.7 (1007), 2019/05/08 12:15:02
//                        $Revision: 1001 $ 
//                        $Date: 2019-04-23 16:56:01 +0200 (Tue, 23 Apr 2019) $
//                        $Author: klaus.malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.message.Message;

import de.knipp.rritoolkit.response.QueueReadResponse;

import static de.knipp.rritoolkit.DenicConstants.NS_MESSAGE;

/**
 *  implementation of the queue read request
 *
 *  @version  $Revision: 1001 $
 *  @author   Klaus Malorny
 */

public class QueueReadRequest extends Request
{
  /** the requested message type */
  
  private Message.Type typeLimitation;
  
  
  
  /**
   *  constructor
   */
  
  public QueueReadRequest ()
  {
    // empty
  }
  
  
  /**
   *  constructor
   *
   *  @param type                     the type of message to read
   */
  
  public QueueReadRequest (Message.Type type)
  {
    typeLimitation = type;
  }
  
  
  /**
   *  return the message type to which the read request is limited
   *
   *  @return                   the type or {@code null} if the request
   *                            is not limited
   */
  
  public Message.Type getTypeLimitation ()
  {
    return typeLimitation;
  }
  
  
  
  /**
   *  set the message type to which the read request shall be limited
   *
   *  @param type               the type or {@code null} if the request
   *                            shall not be limited
   */
  
  public void setTypeLimitation (Message.Type type)
  {
    typeLimitation = type;
  }
  
  
  //--- implementation of DenicRequest -----------------------------------------

  /**
   *  execute the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */
  
  @Override
  public QueueReadResponse execute (Port port)
    throws IOException
  {
    return (QueueReadResponse) super.execute (port);
  }
  
  
  /**
   *  method that needs to be implemented by subclasses for the XML conversion
   *
   *  @param node                 the node where the request shall be added to
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToXml (Node node)
    throws DOMException, EncodeException
  {
    XmlBuilder xb = new XmlBuilder (node);
    
    xb.es (NS_MESSAGE, "queue-read");
    
    if (typeLimitation != null) 
      xb.attr ("msgType", typeLimitation.getDenicToken ());
    
    xb.ee ();
  }
  
  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "queue-read";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    if (typeLimitation != null) 
      list.add ("msgtype", typeLimitation.getDenicToken ());
  }
  

 
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                      the string representation
   */
  
  @Override
  public String toString ()
  {
    return super.toString () + "  type limitation: " + typeLimitation + "\n";
  }
}



